
/*
  Copyright (c) 2009 Dell Inc. by Narendra K <Narendra_K@dell.com>

  libnetdevname library for network device name mapping

  the function takes a pathname to an interface and returns the 
   eth name devicename would point to a ethN name
       0 - success 
      -1 - failure 

  Permission is hereby granted, free of charge, to any person
  obtaining a copy of this software and associated documentation
  files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use,
  copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be
  included in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
  WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
  OTHER DEALINGS IN THE SOFTWARE.
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <net/if.h>

#include "netdevname.h"

#define DEV_PREFIX	"/dev/net/"

int netdev_pathname_to_name(char *devicename, char *kernelname) {
	
	struct stat buf;
	unsigned int ifindex;
	
	/* if the device name passed is a pathname, extract 
	 * the ethN name from it and return the ethN by reference */
	if (strstr(devicename, DEV_PREFIX)) {

	/* if the devicename passed is /dev/net/tun we don't handle it 
 	 * because it does not belong to net class 
 	 * TODO: any other generic way to handle this, what if a new device gets added to /dev/net/ */
		if (!strcmp(devicename, "/dev/net/tun")) {
			fprintf(stderr, "netdev_pathname_to_name: %s does not"
				"belong to net class\n", devicename);
			return -1;
		}

	/* Stat the pathname to get the minor number of the device
	 * node, which happens to be the interface index of the interface */	
		if (!stat(devicename, &buf)) {
			ifindex = minor(buf.st_rdev);
			
			/* Find out the corresponding interface
			 *  name from the interface index */
			if (!if_indextoname(ifindex, kernelname)) {
				fprintf(stderr, "netdev_pathname_to_name: cannot "
					"map ifindex to ifname from %s\n",devicename);
				return -1; 
			}

			printf("%s ---> %s\n",devicename,kernelname);
			return 0;

		} else {
			/* devicename is replaced with eth name only
			 * if stat and if_indextoname succeed */
			fprintf(stderr, "netdev_pathname_to_name:"
				" cannot stat %s\n", devicename);

			return -1;
		}
	}
	else {
		/* if the device name passed does not contain a /dev/net prefix
 		 * then just return what is passed and indicate success */	 
		strncpy(kernelname, devicename, strlen(devicename));
		printf("%s ---> %s\n",devicename,kernelname);
		return 0;
	}
}

void show_pathname_usage(void) {
	fprintf(stdout, 
	"Pathname Usage:\n"	
	"\t/dev/net/ethN\n"
	"\t/dev/net/by-chassis-label/Embedded_NIC_N\n"
	"\t/dev/net/by-mac/xx:xx:xx:xx:xx:xx\n\n");
	return 0;
}
	
